@@ -18,6 +18,7 @@
  * 02111-1307, USA.
  *
  * Authors: William Jon McCann <mccann@jhu.edu>
+ *          Rodrigo Moya <rodrigo@novell.com>
  *
  */
 
@@ -58,8 +59,8 @@
 #define KEY_THEMES       KEY_DIR "/themes"
 
 enum {
-        NAME_COLUMN,
         LABEL_COLUMN,
+        NAME_COLUMN,
         N_COLUMNS
 };
 
@@ -156,6 +157,8 @@ config_get_theme (gboolean *is_writable)
                 name = g_strdup ("__blank-only");
         } else if (mode == GS_MODE_DONT_BLANK) {
                 name = g_strdup ("__disabled");
+        } else if (mode == GS_MODE_RANDOM) {
+                name = g_strdup ("__random");
         } else {
                 GSList *list;
                 list = gconf_client_get_list (client,
@@ -191,6 +194,11 @@ config_set_theme (const char *name)
                 mode = GS_MODE_DONT_BLANK;
         } else if (name && strcmp (name, "__blank-only") == 0) {
                 mode = GS_MODE_BLANK_ONLY;
+        } else if (name && strcmp (name, "__random") == 0) {
+                mode = GS_MODE_RANDOM;
+
+                /* set the themes key to contain all available screensavers */
+                list = gs_job_get_theme_list (job);
         } else {
                 mode = GS_MODE_SINGLE;
                 list = g_slist_append (list, g_strdup (name));
@@ -205,12 +213,45 @@ config_set_theme (const char *name)
                                list,
                                NULL);
 
-        g_slist_foreach (list, (GFunc)g_free, NULL);
+        g_slist_foreach (list, (GFunc) g_free, NULL);
         g_slist_free (list);
 
         g_object_unref (client);
 }
 
+static gboolean
+config_get_lock (gboolean *is_writable)
+{
+        GConfClient *client;
+        gboolean     lock;
+
+        client = gconf_client_get_default ();
+
+        if (is_writable) {
+                *is_writable = gconf_client_key_is_writable (client,
+                                                             KEY_LOCK,
+                                                             NULL);
+        }
+
+        lock = gconf_client_get_bool (client, KEY_LOCK, NULL);
+
+        g_object_unref (client);
+
+        return lock;
+}
+
+static void
+config_set_lock (gboolean lock)
+{
+        GConfClient *client;
+
+        client = gconf_client_get_default ();
+
+        gconf_client_set_bool (client, KEY_LOCK, lock, NULL);
+
+        g_object_unref (client);
+}
+
 static void
 preview_clear (GtkWidget *widget)
 {
@@ -224,9 +265,19 @@ static void
 preview_set_theme (GtkWidget  *widget,
                    const char *theme)
 {
-        GtkWidget *box   = glade_xml_get_widget (xml, "blank_delay_hbox");
+        GtkWidget *delay_box;
+        GtkWidget *lock_box;
+        gboolean   delay_writable;
+        gboolean   lock_writable;
         GError    *error = NULL;
 
+        delay_box = glade_xml_get_widget (xml, "blank_delay_hbox");
+        lock_box = glade_xml_get_widget (xml, "lock_checkbox");
+
+        /* see if the keys are writable */
+        config_get_blank_delay (&delay_writable);
+        config_get_lock (&lock_writable);
+
         if (job) {
                 gs_job_stop (job);
         }
@@ -234,11 +285,45 @@ preview_set_theme (GtkWidget  *widget,
         preview_clear (widget);
 
         if (theme && strcmp (theme, "__disabled") == 0) {
-                gtk_widget_set_sensitive (box, FALSE);
-        } else if (theme && strcmp (theme, "__blank-only") == 0) {
-                gtk_widget_set_sensitive (box, TRUE);
+                if (delay_writable)
+                        gtk_widget_set_sensitive (delay_box, FALSE);
+                if (lock_writable)
+                        gtk_widget_set_sensitive (lock_box, FALSE);
+                
+        } else if ((theme && strcmp (theme, "__blank-only") == 0)) {
+                if (delay_writable)
+                        gtk_widget_set_sensitive (delay_box, TRUE);
+                if (lock_writable)
+                        gtk_widget_set_sensitive (lock_box, TRUE);
+        } else if (theme && strcmp (theme, "__random") == 0) {
+                GSList *themes;
+
+                if (delay_writable)
+                        gtk_widget_set_sensitive (delay_box, TRUE);
+                if (lock_writable)
+                        gtk_widget_set_sensitive (lock_box, TRUE);
+
+                themes = gs_job_get_theme_list (job);
+                if (themes) {
+                        if (!gs_job_set_theme (job, (const char *) themes->data, &error)) {
+                                if (error) {
+                                        g_warning ("Could not set theme: %s", error->message);
+                                        g_error_free (error);
+                                }
+                                return;
+                        }
+                        g_slist_foreach (themes, (GFunc) g_free, NULL);
+                        g_slist_free (themes);
+
+                        gs_job_start (job);
+                }
+                
         } else {
-                gtk_widget_set_sensitive (box, TRUE);
+                if (delay_writable)
+                        gtk_widget_set_sensitive (delay_box, TRUE);
+                if (lock_writable)
+                        gtk_widget_set_sensitive (lock_box, TRUE);
+
                 if (! gs_job_set_theme (job, theme, &error)) {
                         if (error) {
                                 g_warning ("Could not set theme: %s", error->message);
@@ -290,6 +375,12 @@ populate_model (GtkTreeStore *store)
                             NAME_COLUMN, "__blank-only",
                             -1);
 
+        gtk_tree_store_append (store, &iter, NULL);
+        gtk_tree_store_set (store, &iter,
+                            LABEL_COLUMN, "Random",
+                            NAME_COLUMN, "__random",
+                            -1);
+
         gtk_tree_store_append (store, &iter, NULL);
         gtk_tree_store_set (store, &iter,
                             LABEL_COLUMN, NULL,
@@ -385,6 +476,10 @@ compare_theme  (GtkTreeModel *model,
                 return -1;
         else if (strcmp (name_b, "__blank-only") == 0)
                 return 1;
+        else if (strcmp (name_a, "__random") == 0)
+                return -1;
+        else if (strcmp (name_b, "__random") == 0)
+                return 1;
         else if (strcmp (name_a, "__separator") == 0)
                 return -1;
         else if (strcmp (name_b, "__separator") == 0)
@@ -753,6 +848,12 @@ format_value_callback_time (GtkScale *scale,
         return time_to_string_text (value * 60.0);
 }
 
+static void
+lock_checkbox_toggled (GtkToggleButton *button, gpointer user_data)
+{
+        config_set_lock (gtk_toggle_button_get_active (button));
+}
+
 static void
 init_capplet (void)
 {
@@ -762,10 +863,13 @@ init_capplet (void)
         GtkWidget *blank_delay_hscale;
         GtkWidget *blank_delay_hbox;
         GtkWidget *label;
+        GtkWidget *lock_checkbox;
         char      *glade_file;
         char      *path;
+        char      *string;
         gdouble    blank_delay;
         gboolean   is_writable;
+        GConfClient *client;
 
         glade_file = g_build_filename (GLADEDIR, GLADE_XML_FILE, NULL);
         xml = glade_xml_new (glade_file, NULL, PACKAGE);
@@ -792,6 +896,7 @@ init_capplet (void)
         treeview           = glade_xml_get_widget (xml, "savers_treeview");
         blank_delay_hscale = glade_xml_get_widget (xml, "blank_delay_hscale");
         blank_delay_hbox   = glade_xml_get_widget (xml, "blank_delay_hbox");
+        lock_checkbox      = glade_xml_get_widget (xml, "lock_checkbox");
 
         label              = glade_xml_get_widget (xml, "blank_delay_label");
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), blank_delay_hscale);
@@ -805,6 +910,12 @@ init_capplet (void)
         g_signal_connect (blank_delay_hscale, "format-value",
                           G_CALLBACK (format_value_callback_time), NULL);
 
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lock_checkbox), config_get_lock (&is_writable));
+        if (! is_writable)
+                gtk_widget_set_sensitive (lock_checkbox, FALSE);
+        g_signal_connect (lock_checkbox, "toggled",
+                          G_CALLBACK (lock_checkbox_toggled), NULL);
+            
         gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
 
 
@@ -821,6 +932,27 @@ init_capplet (void)
 
         gtk_widget_show_all (dialog);
 
+        /* Update list of themes if using random screensaver */
+        client = gconf_client_get_default ();
+        string = gconf_client_get_string (client, KEY_MODE, NULL);
+        if (string) {
+                int mode;
+                GSList *list;
+
+                gconf_string_to_enum (mode_enum_map, string, &mode);
+                g_free (string);
+
+                if (mode == GS_MODE_RANDOM) {
+                        list = gs_job_get_theme_list (job);
+                        gconf_client_set_list (client, KEY_THEMES, GCONF_VALUE_STRING, list, NULL);
+
+                        g_slist_foreach (list, (GFunc) g_free, NULL);
+                        g_slist_free (list);
+                }
+        }
+
+        g_object_unref (client);
+
         preview_clear (preview);
         gs_job_set_widget (job, preview);
 
