@@ -40,6 +40,58 @@
 
 #include "fusa-manager.h"
 
+/* Profiling stuff adapted from gtkfilechooserdefault */
+
+#undef PROFILE_LOCK_DIALOG
+#ifdef PROFILE_LOCK_DIALOG
+
+#define PROFILE_INDENT 4
+static int profile_indent;
+
+static void
+profile_add_indent (int indent)
+{
+        profile_indent += indent;
+        if (profile_indent < 0)
+                g_error ("You screwed up your indentation");
+}
+
+static void
+_gs_lock_plug_profile_log (const char *func,
+                           int         indent,
+                           const char *msg1,
+                           const char *msg2)
+{
+        char    *str;
+        GTimeVal now;
+
+        if (indent < 0)
+                profile_add_indent (indent);
+
+        g_get_current_time (&now);
+
+        if (profile_indent == 0)
+                str = g_strdup_printf ("MARK %ld.%6.6ld: %s: %s %s %s", now.tv_sec, now.tv_usec, G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+        else
+                str = g_strdup_printf ("MARK %ld.%6.6ld: %s: %*c %s %s %s", now.tv_sec, now.tv_usec, G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+
+        fprintf (stderr, "%s\n", str);
+        g_free (str);
+
+        if (indent > 0)
+                profile_add_indent (indent);
+}
+
+#define profile_start(x, y) _gs_lock_plug_profile_log (G_STRFUNC, PROFILE_INDENT, x, y)
+#define profile_end(x, y)   _gs_lock_plug_profile_log (G_STRFUNC, -PROFILE_INDENT, x, y)
+#define profile_msg(x, y)   _gs_lock_plug_profile_log (NULL, 0, x, y)
+#else
+#define profile_start(x, y)
+#define profile_end(x, y)
+#define profile_msg(x, y)
+#endif
+
+
 enum { 
         AUTH_PAGE = 0,
         SWITCH_PAGE
@@ -202,6 +254,23 @@ switch_user_response (GSLockPlug *plug)
         do_user_switch (plug, display);
 }
 
+static void
+set_status_text (GSLockPlug *plug,
+                 const char *text)
+{
+        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), text);
+}
+
+static void
+set_progress_position (GSLockPlug *plug,
+                       gdouble     fraction,
+                       const char *text)
+{
+        gtk_progress_bar_set_fraction  (GTK_PROGRESS_BAR (plug->priv->progress_bar), fraction);
+        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), text);
+
+}
+
 static void
 gs_lock_plug_response (GSLockPlug *plug,
                        gint        response_id)
@@ -240,12 +309,11 @@ gs_lock_plug_response (GSLockPlug *plug,
         if (response_id == GS_LOCK_PLUG_RESPONSE_OK) {
                 gint current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
 
-                gtk_progress_bar_set_fraction  (GTK_PROGRESS_BAR (plug->priv->progress_bar), 0);
-                gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), " ");
+                set_progress_position (plug, 0, " ");
 
                 if (current_page == 0) {
                         gtk_widget_set_sensitive (plug->priv->password_entry, FALSE);
-                        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), _("Checking password..."));
+                        set_status_text (plug, _("Checking password..."));
 
                         plug->priv->password_check_idle_id = g_idle_add ((GSourceFunc)password_check_idle_cb,
                                                                          plug);
@@ -290,22 +358,18 @@ monitor_progress (GSLockPlug *plug)
         remaining = plug->priv->timeout - elapsed;
 
         fraction = CLAMP ((gdouble)remaining / plug->priv->timeout, 0, 1);
-
-        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (plug->priv->progress_bar), fraction);
-
         secs = remaining / 1000 + 1;
         message = g_strdup_printf (ngettext ("About %ld second left",
                                              "About %ld seconds left", secs),
                                    secs);
-        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar),
-                                   message);
+
+        set_progress_position (plug, fraction, message);
         g_free (message);
 
         if ((remaining <= 0) || (remaining > plug->priv->timeout)) {
                 message = g_strdup (_("Time expired!"));
                 gtk_widget_set_sensitive (plug->priv->password_entry, FALSE);
-                gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar),
-                                           message);
+                set_status_text (plug, message);
                 g_free (message);
 
                 if (plug->priv->response_idle_id == 0)
@@ -365,13 +429,18 @@ restart_monitor_progress (GSLockPlug *plug)
 static void
 gs_lock_plug_show (GtkWidget *widget)
 {
+        profile_start ("start", NULL);
 
+        profile_start ("start", "parent");
         if (GTK_WIDGET_CLASS (parent_class)->show)
                 GTK_WIDGET_CLASS (parent_class)->show (widget);
+        profile_end ("end", "parent");
 
         capslock_update (GS_LOCK_PLUG (widget), is_capslock_on ());
 
         restart_monitor_progress (GS_LOCK_PLUG (widget));
+
+        profile_end ("end", NULL);
 }
 
 static void
@@ -535,7 +604,7 @@ password_check_idle_cb (GSLockPlug *plug)
                                                                       (GSourceFunc)response_idle_cb,
                                                                       plug);
 
-                gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), _("Password check failed!"));
+                set_status_text (plug, _("Password check failed!"));
         }
 
         memset (local_password, '\b', strlen (local_password));
@@ -929,7 +998,7 @@ user_displays_changed_cb (FusaUser           *user,
 
         filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (data->tree));
         model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
-        g_message ("Disp changed");
+
         gtk_list_store_set (GTK_LIST_STORE (model), &data->iter,
                             NAME_COLUMN, name,
                             DISPLAY_NAME_COLUMN, fusa_user_get_display_name (user),
@@ -948,6 +1017,8 @@ populate_model (GSLockPlug   *plug,
         int           icon_size = FACE_ICON_SIZE;
         GtkIconTheme *theme;
 
+        profile_start ("start", NULL);
+
         if (gtk_widget_has_screen (plug->priv->user_treeview))
                 theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (plug->priv->user_treeview));
         else
@@ -969,7 +1040,12 @@ populate_model (GSLockPlug   *plug,
                             -1);
 #endif
 
+        profile_start ("start", "FUSA list users");
+        if (! plug->priv->fusa_manager)
+                plug->priv->fusa_manager = fusa_manager_ref_default ();
+
         users = fusa_manager_list_users (plug->priv->fusa_manager);
+        profile_end ("end", "FUSA list users");
 
         while (users) {
                 FusaUser           *user;
@@ -1010,6 +1086,8 @@ populate_model (GSLockPlug   *plug,
 
                 users = g_slist_delete_link (users, users);
         }
+
+        profile_end ("end", NULL);
 }
 
 static int
@@ -1114,6 +1192,8 @@ setup_treeview (GSLockPlug *plug)
         GtkCellRenderer   *renderer;
         GtkTreeModel      *filter;
 
+        profile_start ("start", NULL);
+
         store = gtk_list_store_new (N_COLUMNS,
                                     G_TYPE_STRING,
                                     G_TYPE_STRING,
@@ -1159,6 +1239,15 @@ setup_treeview (GSLockPlug *plug)
         gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
                                               0,
                                               GTK_SORT_ASCENDING);
+        profile_end ("end", NULL);
+}
+
+static gboolean
+setup_treeview_idle (GSLockPlug *plug)
+{
+        setup_treeview (plug);
+
+        return FALSE;
 }
 
 static const char *
@@ -1175,6 +1264,46 @@ get_user_display_name (void)
         return name;
 }
 
+static void
+label_set_bold (GtkLabel *label)
+{
+        PangoAttrList        *pattrlist;
+        PangoAttribute       *attr;
+
+        pattrlist = pango_attr_list_new ();
+        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+        attr->start_index = 0;
+        attr->end_index = G_MAXINT;
+        pango_attr_list_insert (pattrlist, attr);
+
+        gtk_label_set_attributes (label, pattrlist);
+
+        pango_attr_list_unref (pattrlist);
+}
+
+static void
+label_set_big (GtkLabel *label)
+{
+        PangoAttrList        *pattrlist;
+        PangoAttribute       *attr;
+
+        pattrlist = pango_attr_list_new ();
+
+        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+        attr->start_index = 0;
+        attr->end_index = G_MAXINT;
+        pango_attr_list_insert (pattrlist, attr);
+
+        attr = pango_attr_scale_new (1.2);
+        attr->start_index = 0;
+        attr->end_index = G_MAXINT;
+        pango_attr_list_insert (pattrlist, attr);
+
+        gtk_label_set_attributes (label, pattrlist);
+
+        pango_attr_list_unref (pattrlist);
+}
+
 static void
 gs_lock_plug_init (GSLockPlug *plug)
 {
@@ -1185,13 +1314,13 @@ gs_lock_plug_init (GSLockPlug *plug)
         GtkWidget            *table;
         GtkWidget            *label;
         int                   font_size;
-        PangoAttrList        *pattrlist;
-        PangoAttribute       *attr;
         PangoFontDescription *fontdesc;
 
+        profile_start ("start", NULL);
+
         plug->priv = GS_LOCK_PLUG_GET_PRIVATE (plug);
 
-        plug->priv->fusa_manager = fusa_manager_ref_default ();
+        plug->priv->fusa_manager = NULL;
 
         /* Dialog emulation */
 
@@ -1218,6 +1347,8 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         /* Page 1 */
 
+        profile_start ("start", "page one");
+
         hbox = gtk_hbox_new (FALSE, 12);
         gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), hbox, NULL);
 
@@ -1225,26 +1356,32 @@ gs_lock_plug_init (GSLockPlug *plug)
         gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0);
         gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 
-        vbox = gtk_vbox_new (FALSE, 12);
+        vbox = gtk_vbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
+        /* should we make this a gconf preference? */
+        if (1) {
+                char      *str;
+                GtkWidget *vbox2;
+
+                /* translators: %s is a computer hostname */
+                str = g_strdup_printf (_("Welcome to %s"), g_get_host_name ());
+                widget = gtk_label_new (str);
+                gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
+                gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
+                g_free (str);
+
+                label_set_big (GTK_LABEL (widget));
+
+                vbox2 = gtk_vbox_new (FALSE, 0);
+                gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 5);
+        }
+
         widget = gtk_label_new (_("Enter a password to unlock the screen"));
         gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
         gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
 
-        pattrlist = pango_attr_list_new ();
-        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        font_size = pango_font_description_get_size (widget->style->font_desc);
-        attr = pango_attr_size_new (font_size * 1.2);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);
+        label_set_bold (GTK_LABEL (widget));
 
         hbox = gtk_hbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -1252,7 +1389,7 @@ gs_lock_plug_init (GSLockPlug *plug)
         widget = gtk_label_new ("");
         gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 
-        table = gtk_table_new (3, 2, FALSE);
+        table = gtk_table_new (4, 2, FALSE);
         gtk_table_set_row_spacings (GTK_TABLE (table), 6);
         gtk_table_set_col_spacings (GTK_TABLE (table), 6);
         gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
@@ -1291,8 +1428,12 @@ gs_lock_plug_init (GSLockPlug *plug)
         gtk_label_set_mnemonic_widget (GTK_LABEL (password_label),
                                        plug->priv->password_entry);
 
+        profile_end ("end", "page one");
+
         /* Page 2 */
 
+        profile_start ("start", "page two");
+
         hbox = gtk_hbox_new (FALSE, 12);
         gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), hbox, NULL);
 
@@ -1307,19 +1448,7 @@ gs_lock_plug_init (GSLockPlug *plug)
         gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
         gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
 
-        pattrlist = pango_attr_list_new ();
-        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        font_size = pango_font_description_get_size (widget->style->font_desc);
-        attr = pango_attr_size_new (font_size * 1.2);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);
+        label_set_bold (GTK_LABEL (widget));
 
         widget = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget),
@@ -1333,15 +1462,16 @@ gs_lock_plug_init (GSLockPlug *plug)
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (plug->priv->user_treeview), FALSE);
         gtk_container_add (GTK_CONTAINER (widget), plug->priv->user_treeview);
 
-        setup_treeview (plug);
+        g_idle_add ((GSourceFunc)setup_treeview_idle, plug);
+
+        profile_end ("end", "page two");
 
         /* Progress bar */
 
         plug->priv->progress_bar = gtk_progress_bar_new ();
-        gtk_progress_bar_set_fraction  (GTK_PROGRESS_BAR (plug->priv->progress_bar), 1.0);
         gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (plug->priv->progress_bar),
                                           GTK_PROGRESS_RIGHT_TO_LEFT);
-        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), " ");
+        set_progress_position (plug, 1.0, " ");
 
         gtk_box_pack_start (GTK_BOX (plug->vbox), plug->priv->progress_bar,
                             FALSE, FALSE, 0);
@@ -1354,6 +1484,8 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         /* Buttons */
 
+        profile_start ("start", "buttons");
+
         plug->priv->switch_button = gtk_button_new ();
         gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->switch_button), FALSE);
 
@@ -1393,6 +1525,8 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         gs_lock_plug_set_default_response (plug, GS_LOCK_PLUG_RESPONSE_OK);
 
+        profile_end ("end", "buttons");
+
         gtk_widget_show_all (plug->vbox);
 
         if (! plug->priv->logout_enabled)
@@ -1437,6 +1571,8 @@ gs_lock_plug_init (GSLockPlug *plug)
                                   G_CALLBACK (button_mnemonic_activate_click),
                                   plug->priv->ok_button);
         }
+
+        profile_end ("end", NULL);
 }
 
 static void
@@ -1451,7 +1587,8 @@ gs_lock_plug_finalize (GObject *object)
 
         g_return_if_fail (plug->priv != NULL);
 
-        g_object_unref (plug->priv->fusa_manager);
+        if (plug->priv->fusa_manager)
+                g_object_unref (plug->priv->fusa_manager);
 
         if (plug->priv->password_check_idle_id > 0) {
                 g_source_remove (plug->priv->password_check_idle_id);
